/*
  Line In Control Plugin   version 1.0.0

  Copyright (C) 2003 Artur Dorochowicz

  This program  is free   software. You  can redistribute   it and/or  modify it
  under the terms of the  GNU General Public License  as published by the   Free
  Software Foundation. Either  version 2  of the  License, or  (at your  option)
  any  later version.

  This program is distributed  in the hope that  it will be useful,  but WITHOUT
  ANY WARRANTY. Without even the implied warranty of MERCHANTABILITY or  FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  All used names are the property of their respective owners.
  PowerPro and plugin template are Copyright (C) by Bruce Switzer.
*/

#include <windows.h>
#include <mmsystem.h>
#include <stdlib.h>

//---DEFINITIONS----------------------------------------------------------------
#define MUTE_ON		10
#define MUTE_OFF	20
#define MUTE_STATUS	30
#define SET_VOLUME	40
#define GET_VOLUME	50

//------------------------------------------------------------------------------
typedef struct tagPProServices
  {
   void (*ErrMessage)(LPSTR, LPSTR);
   BOOL (*MatchCaption)(HWND, LPSTR);
   HWND (*FindMatchingWindow)(LPSTR,BOOL);
   BOOL (*IsRolled)(HWND hw);
   BOOL (*IsTrayMinned)(HWND hw);
   void (*GetExeFullPath)(HWND hw, LPSTR szt);
   void (*RollUp)(HWND hw);
   void (*TrayMin)(HWND hw);
   void (*SendKeys)(LPSTR sz);
   BOOL  (*EvalExpr)(LPSTR sz, LPSTR szo);
   void  (*Debug)(LPSTR sz1, LPSTR sz2,LPSTR sz3, LPSTR sz4, LPSTR sz5, LPSTR sz6);
   LPSTR (*AllocTemp)(UINT leng);
   void (*ReturnString)(LPSTR sz, LPSTR* szargs);
   LPSTR (*GetVarAddr)(LPSTR var);
   LPSTR (*SetVar)(LPSTR var, LPSTR val);
   void (*IgnoreNextClip)();
   void (*Show)(HWND h);
   void (*RunCmd)(LPSTR szCmd, LPSTR szParam, LPSTR szWork);
   BOOL (*InsertStringForBar)( LPSTR szStr, LPSTR szCmd);
   void (*ResetFocus)();
  } PPROSERVICES;

//------------------------------------------------------------------------------
BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpvReserved)
{
	return TRUE;
}

//------------------------------------------------------------------------------
void MakeAction (int action, LPSTR * szargs, PPROSERVICES * ppsv)
{
	unsigned int err;
	HMIXER mixerHandle;

	**szargs = '\0';

	err = mixerOpen(&mixerHandle, 0, 0, 0, 0);
	if (err)
	{
		ppsv->ErrMessage("Error! mixerOpen() error.","");
		return;
	}
	else
	{
		MIXERLINE mixerline;
		mixerline.cbStruct = sizeof(MIXERLINE);
		mixerline.dwComponentType = MIXERLINE_COMPONENTTYPE_SRC_LINE;

		if ((err = mixerGetLineInfo((HMIXEROBJ)mixerHandle, &mixerline, MIXER_GETLINEINFOF_COMPONENTTYPE)))
		{
			mixerClose(mixerHandle);
			ppsv->ErrMessage("Error! mixerGetLineInfo() error.","");
			return;
		}
		else
		{
			switch (action)
			{
			case MUTE_ON:
			case MUTE_OFF:
			case MUTE_STATUS:
			{
	 			MIXERCONTROL       mixerControlArray;
				MIXERLINECONTROLS  mixerLineControls;

				mixerLineControls.cbStruct = sizeof(MIXERLINECONTROLS);
				mixerLineControls.dwLineID = mixerline.dwLineID;
				mixerLineControls.cControls = 1;
				mixerLineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_MUTE;
				mixerLineControls.pamxctrl = &mixerControlArray;
				mixerLineControls.cbmxctrl = sizeof(MIXERCONTROL);

				if ((err = mixerGetLineControls((HMIXEROBJ)mixerHandle, &mixerLineControls, MIXER_GETLINECONTROLSF_ONEBYTYPE)))
				{
					mixerClose(mixerHandle);
					ppsv->ErrMessage("Error! mixerGetLineControls() error.", "");
					return;
				}
				else
				{
					switch (action)
					{
					case MUTE_ON:
					{
						MIXERCONTROLDETAILS_UNSIGNED value;
						MIXERCONTROLDETAILS          mixerControlDetails;

						mixerControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerControlDetails.dwControlID = mixerControlArray.dwControlID;
						mixerControlDetails.cChannels = 1;
						mixerControlDetails.cMultipleItems = 0;
						mixerControlDetails.paDetails = &value;
						mixerControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						value.dwValue = 1;

						if ((err = mixerSetControlDetails((HMIXEROBJ)mixerHandle, &mixerControlDetails, MIXER_SETCONTROLDETAILSF_VALUE)))
						{
							mixerClose(mixerHandle);
							ppsv->ErrMessage("Error! mixerSetControlDetails() error.","");
							return;
						}
						break;
					}
					case MUTE_OFF:
					{
						MIXERCONTROLDETAILS_UNSIGNED value;
						MIXERCONTROLDETAILS          mixerControlDetails;

						mixerControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerControlDetails.dwControlID = mixerControlArray.dwControlID;
						mixerControlDetails.cChannels = 1;
						mixerControlDetails.cMultipleItems = 0;
						mixerControlDetails.paDetails = &value;
						mixerControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
						value.dwValue = 0;

						if ((err = mixerSetControlDetails((HMIXEROBJ)mixerHandle, &mixerControlDetails, MIXER_SETCONTROLDETAILSF_VALUE)))
						{
							mixerClose(mixerHandle);
							ppsv->ErrMessage("Error! mixerSetControlDetails() error.","");
							return;
						}
						break;
					}
					case MUTE_STATUS:
					{
						MIXERCONTROLDETAILS_UNSIGNED value;
						MIXERCONTROLDETAILS          mixerControlDetails;

						mixerControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerControlDetails.dwControlID = mixerControlArray.dwControlID;
						mixerControlDetails.cChannels = 1;
						mixerControlDetails.cMultipleItems = 0;
						mixerControlDetails.paDetails = &value;
						mixerControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);

						if ((err = mixerGetControlDetails((HMIXEROBJ)mixerHandle, &mixerControlDetails, MIXER_GETCONTROLDETAILSF_VALUE)))
						{
							mixerClose(mixerHandle);
							ppsv->ErrMessage("Error! mixerGetControlDetails() error.", "");
							return;
						}
						else
						{
							itoa (value.dwValue, *szargs, 10);
						}
						break;
					}
					}
				}
				break;
			}
			case SET_VOLUME:
			case GET_VOLUME:
			{
				MIXERCONTROL       mixerControlArray;
				MIXERLINECONTROLS  mixerLineControls;

				mixerLineControls.cbStruct = sizeof(MIXERLINECONTROLS);
				mixerLineControls.dwLineID = mixerline.dwLineID;;
				mixerLineControls.cControls = 1;
				mixerLineControls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;
				mixerLineControls.pamxctrl = &mixerControlArray;
				mixerLineControls.cbmxctrl = sizeof(MIXERCONTROL);

				if ((err = mixerGetLineControls((HMIXEROBJ)mixerHandle, &mixerLineControls, MIXER_GETLINECONTROLSF_ONEBYTYPE)))
				{
					mixerClose(mixerHandle);
					ppsv->ErrMessage("Error! mixerGetLineControls() error.", "");
					return;
				}
				else
				{
					switch (action)
					{
					case SET_VOLUME:
					{
						MIXERCONTROLDETAILS_UNSIGNED value;
						MIXERCONTROLDETAILS          mixerControlDetails;

						mixerControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerControlDetails.dwControlID = mixerControlArray.dwControlID;
						mixerControlDetails.cChannels = 1;
						mixerControlDetails.cMultipleItems = 0;
						mixerControlDetails.paDetails = &value;
						mixerControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);

						value.dwValue = atoi(*(szargs + 1)) * 655.35;

						if ((err = mixerSetControlDetails((HMIXEROBJ)mixerHandle, &mixerControlDetails, MIXER_SETCONTROLDETAILSF_VALUE)))
						{
							mixerClose(mixerHandle);
							ppsv->ErrMessage("Error! mixerSetControlDetails() error.","");
							return;
						}
						break;
					}
					case GET_VOLUME:
					{
						MIXERCONTROLDETAILS_UNSIGNED value;
						MIXERCONTROLDETAILS          mixerControlDetails;

						mixerControlDetails.cbStruct = sizeof(MIXERCONTROLDETAILS);
						mixerControlDetails.dwControlID = mixerControlArray.dwControlID;
						mixerControlDetails.cChannels = 1;
						mixerControlDetails.cMultipleItems = 0;
						mixerControlDetails.paDetails = &value;
						mixerControlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);

						if ((err = mixerGetControlDetails((HMIXEROBJ)mixerHandle, &mixerControlDetails, MIXER_GETCONTROLDETAILSF_VALUE)))
						{
							mixerClose(mixerHandle);
							ppsv->ErrMessage("Error! mixerGetControlDetails() error.", "");
							return;
						}
						else
						{
							itoa (value.dwValue/655, *szargs, 10);
						}
						break;
					}
					}
				}
				break;
			}
			}
		}
		mixerClose(mixerHandle);
	}
}

//------------------------------------------------------------------------------
_declspec(dllexport) void muteon (LPSTR szv, LPSTR szx, BOOL (*GetVar)(LPSTR, LPSTR), void (*SetVar)(LPSTR, LPSTR), DWORD * pFlags, UINT nargs, LPSTR * szargs, PPROSERVICES * ppsv)
{
	MakeAction (MUTE_ON, szargs, ppsv);
}

//------------------------------------------------------------------------------
_declspec(dllexport) void muteoff (LPSTR szv, LPSTR szx, BOOL (*GetVar)(LPSTR, LPSTR), void (*SetVar)(LPSTR, LPSTR), DWORD * pFlags, UINT nargs, LPSTR * szargs, PPROSERVICES * ppsv)
{
	MakeAction (MUTE_OFF, szargs, ppsv);
}

//------------------------------------------------------------------------------
_declspec(dllexport) void mutestatus (LPSTR szv, LPSTR szx, BOOL (*GetVar)(LPSTR, LPSTR), void (*SetVar)(LPSTR, LPSTR), DWORD * pFlags, UINT nargs, LPSTR * szargs, PPROSERVICES * ppsv)
{
	MakeAction (MUTE_STATUS, szargs, ppsv);
}

//------------------------------------------------------------------------------
_declspec(dllexport) void setvolume (LPSTR szv, LPSTR szx, BOOL (*GetVar)(LPSTR, LPSTR), void (*SetVar)(LPSTR, LPSTR), DWORD * pFlags, UINT nargs, LPSTR * szargs, PPROSERVICES * ppsv)
{
	MakeAction (SET_VOLUME, szargs, ppsv);
}

//------------------------------------------------------------------------------
_declspec(dllexport) void getvolume (LPSTR szv, LPSTR szx, BOOL (*GetVar)(LPSTR, LPSTR), void (*SetVar)(LPSTR, LPSTR), DWORD * pFlags, UINT nargs, LPSTR * szargs, PPROSERVICES * ppsv)
{
	MakeAction (GET_VOLUME, szargs, ppsv);
}

//------------------------------------------------------------------------------
