//note: this code (like the PCRE headers do) uses unsigned ints to index strings so only strings smaller than 2G (possibly more/less with other compilers/architectures) are supported... and since there's no checking, this code's behaviour would be undefined if it's fed larger strings

#define VERSION "1.4.0 28.09.2006"
#define BLOCK_SIZE 256 //arbitrary value (supposed to be a power of two), reduce to reduce memory footprint or increase to improve performance (it would probably take extreme values to make a substantial difference)
#define MAX_SUBMATCHES 99 //don't set this higher than 32767
#define BACKREFS_EXPR "\\\\+([0-9]{1,2})" //what follows the backslashes should be in the only group of the expression and should match any number between 0 and MAX_SUBMATCHES

#include "pcre.h" //needed by PCRE6 because the POSIX library doesn't work independently anymore but pcreposix.h is also neeeded as this code uses the POSIX syntax
#include "pcreposix.h"
//to use another POSIX-like regex library, comment out the pcreversion service and consider adding the REG_EXTENDED flag to the last parameter of the regcomp function call

#include <windows.h>

static HWND g_hwndPowerPro;

 typedef struct tagPProServices {
void (*ErrMessage)(LPSTR, LPSTR);
BOOL (*MatchCaption)(HWND, LPSTR);
HWND (*FindMatchingWindow)(LPSTR,BOOL);
BOOL (*IsRolled)(HWND hw);
BOOL (*IsTrayMinned)(HWND hw);
void (*GetExeFullPath)(HWND hw, LPSTR szt);
void (*RollUp)(HWND hw);
void (*TrayMin)(HWND hw);
void (*SendKeys)(LPSTR sz);
BOOL (*EvalExpr)(LPSTR sz, LPSTR szo);
void (*Debug)(LPSTR s1, LPSTR s2,LPSTR s3, LPSTR s4, LPSTR s5, LPSTR s6);
LPSTR (*AllocTemp)(UINT leng);
void (*ReturnString)(LPSTR sz, LPSTR* szargs);
LPSTR (*GetVarAddr)(LPSTR var);
LPSTR (*SetVar)(LPSTR var, LPSTR val);
void (*IgnoreNextClip)();
 }
PPROSERVICES;

 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpvReserved) {
if(dwReason==DLL_PROCESS_ATTACH) {g_hwndPowerPro=FindWindow("PowerProMain",NULL);}
else if(dwReason==DLL_PROCESS_DETACH) { /*detaching process*/ }
 return TRUE;}

 _declspec(dllexport) void version(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
strcpy(*szargs,VERSION);
 }

 _declspec(dllexport) void pcreversion(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
char *version=(char*)pcre_version(); //the cast is there to avoid a warning 
strcpy(*szargs,version);
 }

 //reinventing the wheel...
 void mystrncat(char **output,int *outputlen, int *outputsize, char *str, int slen) { //assumes str, slen and *output have been initialized, initializes *outputlen and *outputsize
if (slen==-1) {slen=strlen(str);} //in that case, do a (safe) strcat
if (*output==NULL) {
	//allocating BLOCK_SIZE*2^n instead of BLOCK_SIZE*n (for efficiency)...
	//*outputsize=BLOCK_SIZE*((slen+1+16)/BLOCK_SIZE+1)-16;
	*outputsize=BLOCK_SIZE-16; //leaving 16 bytes under a power of two for overhead (is 16 overkill?)
	while (slen+1>*outputsize) {*outputsize=(*outputsize+16)*2-16;} //slen+1 because strlen doesn't include the terminating null
	*output=malloc(*outputsize);
	*output[0]=0; //no one would forget to initialize that, right?
	*outputlen=0;
} else {
	if (*outputlen+slen+1>*outputsize) {
		//*outputsize=BLOCK_SIZE*((*outputlen+slen+1+16)/BLOCK_SIZE+1)-16;
		*outputsize=(*outputsize+16)*2-16;
		while (*outputlen+slen+1>*outputsize) {*outputsize=(*outputsize+16)*2-16;}
		char *tmpstr=realloc(*output,*outputsize);
		if (tmpstr==NULL) {
			free(*output);
			*output=NULL;
		}
		*output=tmpstr;
	}
}
if (*output!=NULL) {
	strncat(*output,str,slen);
	*outputlen=*outputlen+slen;
}
 }

 //linked list node, will hold data about the replacement string (if it's not empty)
 struct node {
int startsegment;
int lensegment; //if there are no backreference candidates in the replacement string, there will be only one node and this will contain the length of the string
short backref; //backreference number found after the segment, -1 when the segment is ended by an escaped backslash and a number (\\n instead of \n)
struct node *next;
 };

 void freenodes(struct node **firstnode) { //assumes *firstnode is allocated
struct node *ptr;
while (*firstnode!=NULL) {
	ptr=*firstnode;
	*firstnode=ptr->next;
	free(ptr);
}
 }

 void parsereplacement(char *replacement, struct node **firstnode) { //assumes that *firstnode is allocated and set to NULL
struct node *ptr;
ptr=malloc(sizeof(struct node));
if (ptr==NULL) {return;}
*firstnode=ptr;
regex_t rxexpr;
regcomp(&rxexpr,BACKREFS_EXPR,0);
regmatch_t rxoutput[2]; //this assumes there will only be one matched group
char tmpstr[6]; //that should be enough to hold any short
int offset=0;
while (regexec(&rxexpr,replacement+offset,2,rxoutput,0)==0) { //while backreference candidates found
	ptr->startsegment=offset;
	ptr->lensegment=rxoutput[0].rm_so+(rxoutput[1].rm_so-rxoutput[0].rm_so)/2; //adding any escaped backslashes to the segment
	if ((rxoutput[1].rm_so-rxoutput[0].rm_so)%2) { //if there was a odd sequence of backslashes (which would mean that there's an unescaped backslah left)
		tmpstr[0]=0;
		strncat(tmpstr,replacement+offset+rxoutput[1].rm_so,rxoutput[1].rm_eo-rxoutput[1].rm_so); //this assumes the group will not have captured more than 5 characters (BACKREFS_EXPR must be set properly)
		ptr->backref=atoi(tmpstr);
		offset=offset+rxoutput[0].rm_eo;
	} else {
		ptr->backref=-1;
		offset=offset+rxoutput[1].rm_so; //don't move offset all the way so as to put the matched numbers in the next segment
	}
	ptr->next=malloc(sizeof(struct node));
	ptr=ptr->next;
	if (ptr==NULL) {
		//regfree(&rxexpr); //commented because it crashes the PCRE6 from GnuWin32 (but not their PCRE5, perhaps because pcreposix.dll was still self-contained)... this probably (not tested) means that the plugin will cause a memory leak (at least until it's unloaded)!
		freenodes(firstnode);
		return;
	}
}
//adding the final node referencing what's left of the replacement string (if nothing is left, it will reference nothing which would be a bit of a waste)...
ptr->startsegment=offset;
ptr->lensegment=strlen(replacement+offset);
ptr->backref=-1;
ptr->next=NULL;
//regfree(&rxexpr); //crashes PCRE6 (see above)
 }

 byte regexpass(BOOL keepunmatched, char *input, regex_t rxexpr, char *replacement, struct node **firstnode, char **output, int *outputlen, int *outputsize, int* matchend) {
regmatch_t rxoutput[MAX_SUBMATCHES+1];
//if there's no match, the PCRE library will not initialise rxoutput[0], so we need to handle this case...
if (regexec(&rxexpr,input,MAX_SUBMATCHES+1,rxoutput,0)!=0) {
	if (keepunmatched) {
		//dealing with whatever is left of the input string...
		mystrncat(output,outputlen,outputsize,input,-1);
		if (*output==NULL) {return(5);} //Memory_Allocation_Error
	}
	*matchend=-1; //lets the calling function know that there's nothing left to match
	return(2); //No_Match
}
//dealing with whatever is before the match...
if (keepunmatched && rxoutput[0].rm_so>0) {
	mystrncat(output,outputlen,outputsize,input,rxoutput[0].rm_so);
	if (*output==NULL) {return(5);} //Memory_Allocation_Error
}
//replacing (if there's anything in that string)...
if (replacement[0]!=0) {
	short n;
	struct node *ptr;
	ptr=*firstnode;
	while(ptr!=NULL) {
		if (ptr->lensegment!=0) {
			mystrncat(output,outputlen,outputsize,replacement+ptr->startsegment,ptr->lensegment);
			if (*output==NULL) {return(5);} //Memory_Allocation_Error
		}
		if (ptr->backref!=-1) {
			n=ptr->backref;
			if (rxoutput[n].rm_so!=-1) {
				mystrncat(output,outputlen,outputsize,input+rxoutput[n].rm_so,rxoutput[n].rm_eo-rxoutput[n].rm_so);
				if (*output==NULL) {return(5);} //Memory_Allocation_Error
			}
		}
		ptr=ptr->next;
	}
}
if (*(input+rxoutput[0].rm_eo)==0) {*matchend=-1;} //the end of the string is reached
else{*matchend=rxoutput[0].rm_eo;}
if(rxoutput[0].rm_so==0) {return(0);} //Match
return(1); //Partial_Match
 }

 int regexwrapper(BOOL keepunmatched, BOOL matchonce, char *input, char *expression, char *replacement, char **output, int *outputlen) {
struct node *dummynode=NULL;
struct node **firstnode=&dummynode; //yeah, yet another pointer to a pointer (and this time it's really useless: parsereplacement and freenodes could return a pointer to the first node instead!)
if (replacement[0]!=0) {
	parsereplacement(replacement,firstnode);
	if (*firstnode==NULL) {return(5);} //Memory_Allocation_Error
}
regex_t rxexpr;
if (regcomp(&rxexpr,expression,0)!=0) {
	return(8); //Invalid_Pattern
	freenodes(firstnode);
}
int outputsize;
int matchend=0;
byte returncode=0;
if (matchonce) {
returncode=regexpass(keepunmatched,input,rxexpr,replacement,firstnode,output,outputlen,&outputsize,&matchend);
	if (matchend!=-1)  {
		if (keepunmatched) {
			mystrncat(output,outputlen,&outputsize,input+matchend,-1); //adding what's left after the match
			if (*output==NULL) {
				//regfree(&rxexpr); //crashes PCRE6 (see above)
				freenodes(firstnode);
				return(5); //Memory_Allocation_Error
			}
		}
		returncode=1; //matchend!=1 only if there's been a partial match
	}
} else {
	int caret=0;
	byte passresult;
	while(matchend!=-1) {
		passresult=regexpass(keepunmatched,input+caret,rxexpr,replacement,firstnode,output,outputlen,&outputsize,&matchend);
		if (passresult<2) {
			if (matchend==0) { //if there's been a zero-char match, let's prevent infinite loops the way sed (for example) does:
				if (keepunmatched) {
					mystrncat(output,outputlen,&outputsize,input+caret,1);
					if (*output==NULL) {
						//regfree(&rxexpr); //crashes PCRE6 (see above)
						freenodes(firstnode);
						return(5); //Memory_Allocation_Error
					}
				}
				caret++;
				if (input[caret]==0) {matchend=-1;} //ends the while loop if this consumed the last character
			} else {
				caret=caret+matchend;
			}
		}
		if (passresult==2) {
			if (caret==0) {returncode=2;}
			else {returncode=1;}
		} else {
			if (passresult>returncode) {returncode=passresult;}
		}
	}
}
if (*output==NULL) { //regexpass does not guarantee the initialization of ouput, so let's turn it into a proper string (note that this function doesn't guarantee that output will be a proper string when it returns 5 or more)
	*output=malloc(1);
	if (*output==NULL) {
		//regfree(&rxexpr); //crashes PCRE6 (see above)
		freenodes(firstnode);
		return(5); //Memory_Allocation_Error
	}
	*output[0]=0;
}
//regfree(&rxexpr); //crashes PCRE6 (see above)
freenodes(firstnode);
return(returncode);
 }

  _declspec(dllexport) void match(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3 || nargs==4) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	int returncode=regexwrapper(FALSE,TRUE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen);
	char returnstr[2];
	itoa(returncode,returnstr,10);
	strcpy(*szargs,returnstr);
	if (nargs==4 && returncode<5) {(ppsv->SetVar(*(szargs+4),*output));}
	free(*output);
} else {
	strcpy(*szargs,"9");  //Invalid_Input
}
 }

 _declspec(dllexport) void matchg(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3 || nargs==4) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	int returncode=regexwrapper(FALSE,FALSE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen);
	char returnstr[2];
	itoa(returncode,returnstr,10);
	strcpy(*szargs,returnstr);
	if (nargs==4 && returncode<5) {(ppsv->SetVar(*(szargs+4),*output));}
	free(*output);
} else {
	strcpy(*szargs,"9");  //Invalid_Input
}
 }

 _declspec(dllexport) void replace(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3 || nargs==4) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	int returncode=regexwrapper(TRUE,TRUE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen);
	char returnstr[2];
	itoa(returncode,returnstr,10);
	strcpy(*szargs,returnstr);
	if (nargs==4 && returncode<5) {(ppsv->SetVar(*(szargs+4),*output));}
	free(*output);
} else {
	strcpy(*szargs,"9");  //Invalid_Input
}
 }

 _declspec(dllexport) void replaceg(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3 || nargs==4) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	int returncode=regexwrapper(TRUE,FALSE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen);
	char returnstr[2];
	itoa(returncode,returnstr,10);
	strcpy(*szargs,returnstr);
	if (nargs==4 && returncode<5) {(ppsv->SetVar(*(szargs+4),*output));}
	free(*output);
} else {
	strcpy(*szargs,"9");  //Invalid_Input
}
 }

 _declspec(dllexport) void m1(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	if (regexwrapper(FALSE,TRUE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen)<5) {
		char *longstring;
		longstring=(ppsv->AllocTemp)(outputlen+1);
		if (longstring) {
			strcpy(longstring,*output);
			(ppsv->ReturnString)(longstring,szargs);
		}
	}
	free(*output);
}
 }

 _declspec(dllexport) void r1(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	if (regexwrapper(TRUE,TRUE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen)<5) {
		char *longstring;
		longstring=(ppsv->AllocTemp)(outputlen+1);
		if (longstring) {
			strcpy(longstring,*output);
			(ppsv->ReturnString)(longstring,szargs);
		}
	}
	free(*output);
}
 }

 _declspec(dllexport) void mg(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	if (regexwrapper(FALSE,FALSE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen)<5) {
		char *longstring;
		longstring=(ppsv->AllocTemp)(outputlen+1);
		if (longstring) {
			strcpy(longstring,*output);
			(ppsv->ReturnString)(longstring,szargs);
		}
	}
	free(*output);
}
 } 

 _declspec(dllexport) void rg(LPSTR szv, LPSTR szx, BOOL(*GetVar)(LPSTR, LPSTR), void(*SetVar)(LPSTR, LPSTR), DWORD* pFlags, UINT nargs, LPSTR* szargs, PPROSERVICES* ppsv) {
if (nargs==3) {
	char *dummyptr=NULL;
	char **output=&dummyptr;
	int outputlen;
	if (regexwrapper(TRUE,FALSE,*(szargs+1),*(szargs+2),*(szargs+3),output,&outputlen)<5) {
		char *longstring;
		longstring=(ppsv->AllocTemp)(outputlen+1);
		if (longstring) {
			strcpy(longstring,*output);
			(ppsv->ReturnString)(longstring,szargs);
		}
	}
	free(*output);
}
 }

//note: strcpy(*szargs,*output); is much neater than AllocTemp/ReturnString but only works for short strings (up to 530 chars)
